@@ -1,9 +1,9 @@
 import pytest
 
-from .env import H2Conf
+from .env import H2Conf, H2TestEnv
 
 
-@pytest.mark.skip(reason="only in 2.5.x")
+@pytest.mark.skipif(condition=H2TestEnv.is_unsupported, reason="mod_http2 not supported here")
 class TestProxyServerHeader:
 
     @pytest.fixture(autouse=True, scope='class')
